@@ -2,11 +2,13 @@
 
 use {
     crate::{
+        rpc::check_is_at_least_confirmed,
         rpc_pubsub_service::PubSubConfig,
         rpc_subscription_tracker::{
-            AccountSubscriptionParams, LogsSubscriptionKind, LogsSubscriptionParams,
-            ProgramSubscriptionParams, SignatureSubscriptionParams, SubscriptionControl,
-            SubscriptionId, SubscriptionParams, SubscriptionToken,
+            AccountSubscriptionParams, BlockSubscriptionKind, BlockSubscriptionParams,
+            LogsSubscriptionKind, LogsSubscriptionParams, ProgramSubscriptionParams,
+            SignatureSubscriptionParams, SubscriptionControl, SubscriptionId, SubscriptionParams,
+            SubscriptionToken,
         },
     },
     dashmap::DashMap,
@@ -16,15 +18,17 @@ use {
     solana_account_decoder::{UiAccount, UiAccountEncoding},
     solana_client::{
         rpc_config::{
-            RpcAccountInfoConfig, RpcProgramAccountsConfig, RpcSignatureSubscribeConfig,
-            RpcTransactionLogsConfig, RpcTransactionLogsFilter,
+            RpcAccountInfoConfig, RpcBlockSubscribeConfig, RpcBlockSubscribeFilter,
+            RpcProgramAccountsConfig, RpcSignatureSubscribeConfig, RpcTransactionLogsConfig,
+            RpcTransactionLogsFilter,
         },
         rpc_response::{
-            Response as RpcResponse, RpcKeyedAccount, RpcLogsResponse, RpcSignatureResult, RpcVote,
-            SlotInfo, SlotUpdate,
+            Response as RpcResponse, RpcBlockUpdate, RpcKeyedAccount, RpcLogsResponse,
+            RpcSignatureResult, RpcVote, SlotInfo, SlotUpdate,
         },
     },
     solana_sdk::{clock::Slot, pubkey::Pubkey, signature::Signature},
+    solana_transaction_status::UiTransactionEncoding,
     std::{str::FromStr, sync::Arc},
 };
 
@@ -187,6 +191,28 @@ pub trait RpcSolPubSub {
         id: PubSubSubscriptionId,
     ) -> Result<bool>;
 
+    // Subscribe to block data and content
+    #[pubsub(subscription = "blockNotification", subscribe, name = "blockSubscribe")]
+    fn block_subscribe(
+        &self,
+        meta: Self::Metadata,
+        subscriber: Subscriber<Arc<RpcBlockUpdate>>,
+        filter: RpcBlockSubscribeFilter,
+        config: Option<RpcBlockSubscribeConfig>,
+    );
+
+    // Unsubscribe from block notification subscription.
+    #[pubsub(
+        subscription = "blockNotification",
+        unsubscribe,
+        name = "blockUnsubscribe"
+    )]
+    fn block_unsubscribe(
+        &self,
+        meta: Option<Self::Metadata>,
+        id: PubSubSubscriptionId,
+    ) -> Result<bool>;
+
     // Get notification when vote is encountered
     #[pubsub(subscription = "voteNotification", subscribe, name = "voteSubscribe")]
     fn vote_subscribe(&self, meta: Self::Metadata, subscriber: Subscriber<RpcVote>);
@@ -295,6 +321,18 @@ mod internal {
         #[rpc(name = "slotsUpdatesUnsubscribe")]
         fn slots_updates_unsubscribe(&self, id: SubscriptionId) -> Result<bool>;
 
+        // Subscribe to block data and content
+        #[rpc(name = "blockSubscribe")]
+        fn block_subscribe(
+            &self,
+            filter: RpcBlockSubscribeFilter,
+            config: Option<RpcBlockSubscribeConfig>,
+        ) -> Result<SubscriptionId>;
+
+        // Unsubscribe from block notification subscription.
+        #[rpc(name = "blockUnsubscribe")]
+        fn block_unsubscribe(&self, id: SubscriptionId) -> Result<bool>;
+
         // Get notification when vote is encountered
         #[rpc(name = "voteSubscribe")]
         fn vote_subscribe(&self) -> Result<SubscriptionId>;
@@ -475,6 +513,42 @@ impl RpcSolPubSubInternal for RpcSolPubSubImpl {
         self.unsubscribe(id)
     }
 
+    fn block_subscribe(
+        &self,
+        filter: RpcBlockSubscribeFilter,
+        config: Option<RpcBlockSubscribeConfig>,
+    ) -> Result<SubscriptionId> {
+        if !self.config.enable_block_subscription {
+            return Err(Error::new(jsonrpc_core::ErrorCode::MethodNotFound));
+        }
+        let config = config.unwrap_or_default();
+        let commitment = config.commitment.unwrap_or_default();
+        check_is_at_least_confirmed(commitment)?;
+        let params = BlockSubscriptionParams {
+            commitment: config.commitment.unwrap_or_default(),
+            encoding: config.encoding.unwrap_or(UiTransactionEncoding::Base64),
+            kind: match filter {
+                RpcBlockSubscribeFilter::All => BlockSubscriptionKind::All,
+                RpcBlockSubscribeFilter::MentionsAccountOrProgram(key) => {
+                    BlockSubscriptionKind::MentionsAccountOrProgram(param::<Pubkey>(
+                        &key,
+                        "mentions_account_or_program",
+                    )?)
+                }
+            },
+            transaction_details: config.transaction_details.unwrap_or_default(),
+            show_rewards: config.show_rewards.unwrap_or_default(),
+        };
+        self.subscribe(SubscriptionParams::Block(params))
+    }
+
+    fn block_unsubscribe(&self, id: SubscriptionId) -> Result<bool> {
+        if !self.config.enable_block_subscription {
+            return Err(Error::new(jsonrpc_core::ErrorCode::MethodNotFound));
+        }
+        self.unsubscribe(id)
+    }
+
     fn vote_subscribe(&self) -> Result<SubscriptionId> {
         if !self.config.enable_vote_subscription {
             return Err(Error::new(jsonrpc_core::ErrorCode::MethodNotFound));
@@ -539,7 +613,10 @@ mod tests {
         solana_stake_program::stake_state,
         solana_vote_program::vote_state::Vote,
         std::{
-            sync::{atomic::AtomicBool, RwLock},
+            sync::{
+                atomic::{AtomicBool, AtomicU64},
+                RwLock,
+            },
             thread::sleep,
             time::Duration,
         },
@@ -578,8 +655,10 @@ mod tests {
         let bank = Bank::new_for_tests(&genesis_config);
         let blockhash = bank.last_blockhash();
         let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let rpc_subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &Arc::new(AtomicBool::new(false)),
+            max_complete_transaction_status_slot,
             bank_forks.clone(),
             Arc::new(RwLock::new(BlockCommitmentCache::new_for_tests())),
             OptimisticallyConfirmedBank::locked_from_bank_forks_root(&bank_forks),
@@ -705,7 +784,11 @@ mod tests {
         let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
 
         let mut io = IoHandler::<()>::default();
-        let subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(bank_forks));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
+        let subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(
+            max_complete_transaction_status_slot,
+            bank_forks,
+        ));
         let (rpc, _receiver) = rpc_pubsub_service::test_connection(&subscriptions);
 
         io.extend_with(rpc.to_delegate());
@@ -756,9 +839,10 @@ mod tests {
         let bank0 = bank_forks.read().unwrap().get(0).unwrap().clone();
         let bank1 = Bank::new_from_parent(&bank0, &Pubkey::default(), 1);
         bank_forks.write().unwrap().insert(bank1);
-
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let rpc_subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &Arc::new(AtomicBool::new(false)),
+            max_complete_transaction_status_slot,
             bank_forks.clone(),
             Arc::new(RwLock::new(BlockCommitmentCache::new_for_tests_with_slots(
                 1, 1,
@@ -873,9 +957,10 @@ mod tests {
         let bank0 = bank_forks.read().unwrap().get(0).unwrap().clone();
         let bank1 = Bank::new_from_parent(&bank0, &Pubkey::default(), 1);
         bank_forks.write().unwrap().insert(bank1);
-
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let rpc_subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &Arc::new(AtomicBool::new(false)),
+            max_complete_transaction_status_slot,
             bank_forks.clone(),
             Arc::new(RwLock::new(BlockCommitmentCache::new_for_tests_with_slots(
                 1, 1,
@@ -963,7 +1048,11 @@ mod tests {
         ))));
 
         let mut io = IoHandler::<()>::default();
-        let subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(bank_forks));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
+        let subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(
+            max_complete_transaction_status_slot,
+            bank_forks,
+        ));
         let (rpc, _receiver) = rpc_pubsub_service::test_connection(&subscriptions);
 
         io.extend_with(rpc.to_delegate());
@@ -1007,8 +1096,10 @@ mod tests {
         let bob = Keypair::new();
 
         let exit = Arc::new(AtomicBool::new(false));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let rpc_subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &exit,
+            max_complete_transaction_status_slot,
             bank_forks.clone(),
             Arc::new(RwLock::new(BlockCommitmentCache::new_for_tests())),
             OptimisticallyConfirmedBank::locked_from_bank_forks_root(&bank_forks),
@@ -1058,9 +1149,10 @@ mod tests {
 
         let exit = Arc::new(AtomicBool::new(false));
         let block_commitment_cache = Arc::new(RwLock::new(BlockCommitmentCache::new_for_tests()));
-
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &exit,
+            max_complete_transaction_status_slot,
             bank_forks.clone(),
             block_commitment_cache,
             OptimisticallyConfirmedBank::locked_from_bank_forks_root(&bank_forks),
@@ -1128,7 +1220,11 @@ mod tests {
         let GenesisConfigInfo { genesis_config, .. } = create_genesis_config(10_000);
         let bank = Bank::new_for_tests(&genesis_config);
         let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
-        let rpc_subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(bank_forks));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
+        let rpc_subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(
+            max_complete_transaction_status_slot,
+            bank_forks,
+        ));
         let (rpc, mut receiver) = rpc_pubsub_service::test_connection(&rpc_subscriptions);
         rpc.slot_subscribe().unwrap();
 
@@ -1156,7 +1252,11 @@ mod tests {
         let GenesisConfigInfo { genesis_config, .. } = create_genesis_config(10_000);
         let bank = Bank::new_for_tests(&genesis_config);
         let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
-        let rpc_subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(bank_forks));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
+        let rpc_subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(
+            max_complete_transaction_status_slot,
+            bank_forks,
+        ));
         let (rpc, mut receiver) = rpc_pubsub_service::test_connection(&rpc_subscriptions);
         let sub_id = rpc.slot_subscribe().unwrap();
 
@@ -1198,8 +1298,10 @@ mod tests {
         // Setup Subscriptions
         let optimistically_confirmed_bank =
             OptimisticallyConfirmedBank::locked_from_bank_forks_root(&bank_forks);
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &exit,
+            max_complete_transaction_status_slot,
             bank_forks,
             block_commitment_cache,
             optimistically_confirmed_bank,
@@ -1228,7 +1330,11 @@ mod tests {
         let GenesisConfigInfo { genesis_config, .. } = create_genesis_config(10_000);
         let bank = Bank::new_for_tests(&genesis_config);
         let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
-        let rpc_subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(bank_forks));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
+        let rpc_subscriptions = Arc::new(RpcSubscriptions::default_with_bank_forks(
+            max_complete_transaction_status_slot,
+            bank_forks,
+        ));
         let (rpc, _receiver) = rpc_pubsub_service::test_connection(&rpc_subscriptions);
         let sub_id = rpc.vote_subscribe().unwrap();
 
